    # Scripts combined and modified by Joshua Driggs (ZapWizard at gmail.com) using code from google and other sources.
	# Thanks to Chris Fuller for helping to package these scripts.
    # Script will now search your entire model for edges to remove, and any lines that need mending.
    # This script will also remove any material that is on a revesed face.
    # These changes were made specificaly to clean up VMF models for export to the Source Hammer editor from Valve Software.

    require 'sketchup.rb'

    ## Menu and GUI code
    #-----------------------------------------------------------------------------
module SU_Source_tools_cleanup
    def SU_Source_tools_cleanup.variables
      @group_list = []
      @component_list = []
      @delete_these = []
      @model_list = Sketchup.active_model.entities
      @badtextures = []
	  @connected_faces=[] #Initalize table
    end

    def SU_Source_tools_cleanup.delete_edges
      variables
      traverse_model_entities
	  
	  orient_faces #Clean up reversed faces
	  
      # The edges that need to be deleted have all been added to the
      # @delete_these list, so erase those entities from the
      # active SketchUp model.
      #Report the number of deleted edges.
      total = @delete_these.length
      badtextotal = @badtextures.length
      Sketchup.active_model.entities.erase_entities(@delete_these)
      UI.messagebox("#{total} lines deleted.
      There were #{badtextotal} incorrectly applied textures.")
    end


    def SU_Source_tools_cleanup.edge_can_be_removed?(item)
		faces = item.faces
		# Delete the edge if it doesn't contribute to a face
		if faces.length == 0 then @delete_these.push(item) end
		# If it contributes to more than one face, it can not be removed
		return false if faces.length != 2;
		# It must contribute to only 2 faces to be a canidate.
		# If someone can think of a situation where an edge could
		# contribute to more than 2 faces and still be a canidate
		# for removal, please let me know.
		normal0 = faces[0].normal
		normal1 = faces[1].normal
		dot = normal0.dot normal1
             
		# What should the tolerance be on the dot product
		# for two faces to be co-planar?

		# Check for two faces pointing in the same direction
		if dot > 0.999999999999999999
			# Check that the materials are the same
			if same_side_material?(faces[0],faces[1]) then @delete_these.push(item) end
		end
          # Check for co-planar faces, but reversed normals
		if dot < -0.999999999999999999
			# Check that the material are the same
			if opposite_side_material?(faces[0],faces[1]) then @delete_these.push(item) end
		end
	end

	
	def SU_Source_tools_cleanup.same_side_material?(face1,face2)
		face1.material == face2.material and face1.back_material == face2.back_material
	end
	
	
	def SU_Source_tools_cleanup.opposite_side_material?(face1,face2)
		face1.material == face2.back_material and face1.back_material == face2.material
	end

    def SU_Source_tools_cleanup.traverse_model_entities
    # @model_list will not be empty until all of the groups, components
    # and edges have been traversed.
       while @model_list != []
       @model_list.each do |item|
          # Determine if the entity is an edge, group, face, or component.
          type = item.typename
          case type
				when "Edge"
					# Determine if the edge should be deleted, and add it to the
					# @delete_these list.
					edge_can_be_removed?(item)
					repair_broken_lines(item)
				when "Group"
					# Add each of the entities in the group into the group_list
					# array.
					item.entities.each do |entity|
						@group_list.push entity
					end
					# Check to see if a material was universally applied to a group
					if item.material != nil then
						@badtextures.push item
						item.material = nil
					end
				when "ComponentInstance"
					# .entities can be called on a Component Definition, but not on
					# a Component Instance.
					# Use the .definition method to access the Component Definition
					# that the instance belongs to.
					item.definition.entities.each do |entity|
						# Add each of the entities in the component into the
						# component_list array.
						@component_list.push entity
					end
					# Check to see if a material was universally applied to a component
					if item.material != nil then
						@badtextures.push item
						item.material = nil
					end
				when "Face"
					#Check all faces for materials applied to a reversed face, and remove the material.
					#Remove backside materials
					if item.back_material != nil then
						@badtextures.push item
						item.back_material = nil
					end
					
					#Make a list of faces that are connected to other faces
					item.all_connected.each do |entity| #For each entity connected to the face
						if entity.class==Sketchup::Face then #Is the connected entity a face?
							entity.edges.each do |edge| #For each edge in the face
								if edge.faces[1] #Does the edge have another face connected to it?
									@connected_faces.push item #If so add the face to the list of faces to check
								break ### found one ###v1.1
								end#if
							end#do
						end#if
					end#each
					
				end#when
        end#do
        # Update the @model_list so it contains only the entities that were
        # embedded in groups or components. These entities haven't been
        # counted yet.
        @model_list = @group_list + @component_list
        # Clear out the group and component lists so they're ready for the
        # next level of sub-groups/components.
        @group_list = []
        @component_list =[]
      end

    end
end




    # The Following script repairs and edge with two lines that meet and create a face, but are split in two or move segments.
    #The script was originally written by Carlo Roosen and modified for use across all models.

    def SU_Source_tools_cleanup.repair_broken_lines(item)
      #model = Sketchup.active_model
      #define everything as one undo operation
      #model.start_operation "Repair Broken Lines"
      #trunk = Array.new
      #ss = model.selection
      #search all edges in selection set
      #for e in ss
      #if (e.kind_of?(Sketchup::Edge))
      p=item.parent
      #take both vertices
      for v in item.vertices
       #take all edges connected to v (including e)
        es=v.edges
       #when two edges are connected
       if (es.length==2)
          vec1 = es[0].line[1]
          vec2 = es[1].line[1]
          #and both are in the same direction
          if (vec1.parallel?(vec2))
             #then the two lines can be repaired
             #this is done by generating a new line from the vertex to a random point
             l = p.entities.add_line(v.position, Geom::Point3d.new(3.45,6.78,9.12))
             ##and put that line in a trunk
             @delete_these.push(l)
           end
        end
      end

      #erase all lines in trunk
      #total = trunk.length
      #for l in trunk
      #  l.erase!
      #end
      #model.commit_operation
      #UI.messagebox ("#{total} lines repared")
   


	#-----------------------------------------------------------------------
	#Copyright 2007 TIG: Permission to use, copy, modify, and distribute 
	#this software for any purpose and without fee is hereby granted, 
	#provided this notice appears in any copies.
	#THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
	#WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF 
	#MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
	#-----------------------------------------------------------------------
	#It matches the orientation of a face to any connected 'co-edged' faces.
	#It mimics SketchUp's "Orient Faces" Tool.
	#It can sometimes produce slightly different results when more than two 
	#connected faces share a common edge.  Its results are equally logical, 
	#and who's to say how they should orient anyway ?
	#You can call it from other scripts thus:
	# face.orient_faces
	#-----------------------------------------------------------------------
	#Version:	1.0	20071215	First Issue.
	#            1.1 20080706    Algorithms improved etc - much faster.
	#-----------------------------------------------------------------------

	def SU_Source_tools_cleanup.orient_faces
		#@connected_faces is now a list of faces where at least one edge is connected to a reversed face
		@connected_faces.uniq! ###v1.1 #Remove all duplicate entries from the list of faces
		@awaiting_faces=@connected_faces #Copy list of faces to list of ones left to be processed.
		@done_faces=[] #Make a list of faces that have been processed
		###
		while @awaiting_faces != [] #While the list of awaiting faces has faces
		  @awaiting_faces.each do |face| #For each face in the list of awaiting faces
			if not @done_faces.include?(face) #First check if this face has been processed already
				if face.class==Sketchup::Face then #Check if the face is a valid face
					@face=face	#Store current face
					face_flip #Fun flip
				end#if
			end#if
		  end#do
		end#while
	end#def

 
	def SU_Source_tools_cleanup.face_flip ###v1.1
		@awaiting_faces=@awaiting_faces-[@face]
		@face.edges.each{|edge|
		  rev1=edge.reversed_in?(@face)
		  @common_faces=edge.faces-[@face]
		  @common_faces.each{|face|
			rev2=edge.reversed_in?(face)
			face.reverse! if @awaiting_faces.include?(face) and rev1==rev2
			@awaiting_faces=@awaiting_faces-[face]
		  }
		}
		@done_faces<<@face
	 end#def
 
	
	
end
	
	
	
	
	
	
#----------Menu system----------
	
	if !file_loaded?('source_sutools') then
		$source_sutools_menu = UI.menu("Plugins").add_submenu("* Source SU Tools *")
	end
	if !file_loaded?(__FILE__) then
		$source_sutools_menu.add_item("Clean up VMF model") { SU_Source_tools_cleanup.delete_edges  }
	end

file_loaded('source_sutools')
file_loaded(__FILE__)
		

    #-----------------------------------------------------------------------------

    # Copyright 2005, John H. Aughey (jha@aughey.com)
    # Copyright 2005, Carlo Roosen

    # Permission to use, copy, modify, and distribute this software for
    # any purpose and without fee is hereby granted, provided that the above
    # copyright notice appear in all copies.

    # THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
    # IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
    # WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
    #-----------------------------------------------------------------------------
